__author__ = "Henry Ginn"
__date__ = "2023/06/05"
__version__ = "1.0"

from .loaddefaults import LoadDefaults as load
from .processkwargs import ProcessKwargs as kwargs
from .inherit import inherit
from .docs import docs

docs()
